/*
 * JBoss, Home of Professional Open Source
 * Copyright 2006, JBoss Inc., and others contributors as indicated
 * by the @authors tag. All rights reserved.
 * See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 * This copyrighted material is made available to anyone wishing to
 * use, modify, copy, or redistribute it subject to the terms and
 * conditions of the GNU Lesser General Public License, v. 2.1.
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT A WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public
 * License v.2.1 along with this distribution; if not, write to the
 * Free software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * (C) 2005-2006,
 * @author JBoss Inc.
 */

package org.hornetq.javaee.example;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;
import java.util.Date;
import java.util.Properties;

import javax.jms.DeliveryMode;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageListener;
import javax.jms.ObjectMessage;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

//import org.jboss.soa.esb.actions.StoreMessageToFile;

public class SendJMSMessage implements MessageListener {
	QueueConnection conn;
	QueueSession session;
	Queue que;
	InitialContext iniCtx;
	MessageConsumer consumer;
	//Queue replyQue;
	int i = 100000;
	boolean processFinished = false;

	long starttime = 0;
	public SendJMSMessage() throws Exception {
		setupConnection();
		// String fileContent = sm.readAsciiFile("/SampleOrder.xml");
		DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory
				.newInstance();
		InputStream inputStream = new FileInputStream(
				new File(
						"C:/jboss/jboss-5.1.0.GA/jbm-examples/saveetha/SampleOrder.xml"));
		org.w3c.dom.Document doc = documentBuilderFactory.newDocumentBuilder()
				.parse(inputStream);
		StringWriter stw = new StringWriter();
		Transformer serializer = TransformerFactory.newInstance()
				.newTransformer();
		serializer.transform(new DOMSource(doc), new StreamResult(stw));
		String fileContent = stw.toString();
		System.out.println("---------------------------------------------");
		System.out.println(fileContent);
		System.out.println("---------------------------------------------");
		sendAMessage(fileContent);
	}

	public void setupConnection() throws JMSException, NamingException {
		Properties properties1 = new Properties();
		properties1.put(Context.INITIAL_CONTEXT_FACTORY,
				"org.jnp.interfaces.NamingContextFactory");
		properties1.put(Context.URL_PKG_PREFIXES,
				"org.jboss.naming:org.jnp.interfaces");
		properties1.put(Context.PROVIDER_URL, "jnp://localhost:1099");
		iniCtx = new InitialContext(properties1);

		Object tmp = iniCtx.lookup("/ConnectionFactory");
		QueueConnectionFactory qcf = (QueueConnectionFactory) tmp;
		conn = qcf.createQueueConnection();
		que = (Queue) iniCtx.lookup("queue/Roundtrip_HornetQ");

		//replyQue = (Queue) iniCtx.lookup("queue/Roundtrip_Reply_HornetQ");

		session = conn.createQueueSession(false, Session.DUPS_OK_ACKNOWLEDGE);
		conn.start();
		System.out.println("Connection Started");
	}

	public void stop() throws JMSException, NamingException {
		conn.stop();
		iniCtx.close();
		session.close();
		conn.close();
	}

	public void sendAMessage(String msg) throws Exception {
		QueueSender send = session.createSender(que);
		send.setDisableMessageID(true);
		send.setDisableMessageTimestamp(true);

		Queue temporaryQueue = session.createTemporaryQueue();
		consumer = session.createConsumer(temporaryQueue);
		consumer.setMessageListener(this);

		ObjectMessage tm = session.createObjectMessage(msg);
		tm.setJMSReplyTo(temporaryQueue);

		starttime = new Date().getTime();

		System.out.println("Message send start time =" + new Date());
		while (i > 0) {
			// System.out.println("into while loop");
			send.send(tm);
			i = i - 1;
		}

		/*
		 * Send a non-text control message indicating end of messages.
		 */
		Message m = session.createMessage();
		m.setJMSReplyTo(temporaryQueue);
		send.send(m);

		long endtime = new Date().getTime();
		System.out.println("########## " + i + " messages sent in "
				+ (endtime - starttime) / 1000 + " seconds.");
		send.close();

		sleepThread();
	}

	public void sleepThread() throws Exception {
		Thread.sleep(5000);
		if (processFinished) {
			stop();
		} else {
			sleepThread();
		}
	}

	public static void main(String args[]) throws Exception {
		new SendJMSMessage();
	}

	public void onMessage(Message message) {
		String messageText = null;
		try {
			if (message instanceof TextMessage) {
				TextMessage textMessage = (TextMessage) message;
				messageText = textMessage.getText();
				System.out.println("messageText = " + messageText);
			} else {
				try {
					long endtime = new Date().getTime();
					System.out.println(" Last message reply received "
						+ (endtime - starttime) / 1000 + " seconds.");
					processFinished = true;
					//stop();
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		} catch (JMSException e) {
			// Handle the exception appropriately
		}
	}

}
