/**
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.hornetq.javaee.example.server;

import javax.annotation.Resource;
import javax.ejb.ActivationConfigProperty;
import javax.ejb.MessageDriven;
import javax.ejb.MessageDrivenContext;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;
import javax.jms.*;
import javax.transaction.UserTransaction;
import javax.jms.ObjectMessage;
import org.jboss.ejb3.annotation.ResourceAdapter;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.util.Date;

/**
 * @author <a href="mailto:andy.taylor@jboss.org">Andy Taylor</a>
 */
@MessageDriven(name = "MDB_BMTExample", activationConfig = { @ActivationConfigProperty(propertyName = "destinationType", propertyValue = "javax.jms.Queue"),
                                                            @ActivationConfigProperty(propertyName = "destination", propertyValue = "queue/Roundtrip_HornetQ"),
                                                            @ActivationConfigProperty(propertyName = "acknowledgeMode", propertyValue = "Dups-ok-acknowledge") })
@TransactionManagement(value = TransactionManagementType.BEAN)
@ResourceAdapter("hornetq-ra.rar")

public class MDB_BMTExample implements MessageListener {
	public void onMessage(Message m) {
		if (m instanceof ObjectMessage) {
			businessLogic(m);
		} else {
			System.out.println("End of all messages");
			businessLogicLastMsg(m);
		}
	}

	private void businessLogicLastMsg(Message m) {
		Connection conn = null;
		Session session = null;
		try {
			InitialContext ic = new InitialContext();
			ConnectionFactory cf = (ConnectionFactory) ic.lookup("java:/JmsXA");
			ic.close();

			conn = cf.createConnection();
			conn.start();
			session = conn.createSession(false, Session.DUPS_OK_ACKNOWLEDGE);

			Destination replyTo = m.getJMSReplyTo();
			//System.out.println("Reply to Destination is " + replyTo);

			MessageProducer producer = session.createProducer(replyTo);
			Message m1 = session.createMessage();
			System.out.println("*** Last mesasge produced ***"+m1+" "+new Date());
			producer.send(m1);

			producer.close();
			ic.close();
		} catch (Exception e) {
			e.printStackTrace();
			System.out.println("The Message Driven Bean failed!");
		} finally {
			if (conn != null) {
				try {
					conn.close();
				} catch (Exception e) {
					System.out.println("Could not close the connection!" + e);
				}
			}
		}
	}

	private void businessLogic(Message m) {
		Connection conn = null;
		Session session = null;

		try {
			Object msgObj = null;
			msgObj = ((ObjectMessage) m).getObject();

			System.out.println("message received");
			String text = "Hello";

			// flip the string
			String result = "";
			for (int i = 0; i < text.length(); i++) {
				result = text.charAt(i) + result;
			}

			InitialContext ic = new InitialContext();
			ConnectionFactory cf = (ConnectionFactory) ic.lookup("java:/JmsXA");
			ic.close();

			conn = cf.createConnection();
			conn.start();
			session = conn.createSession(false, Session.DUPS_OK_ACKNOWLEDGE);

			Destination replyTo = m.getJMSReplyTo();
			//System.out.println("Reply to Destination is " + replyTo);

			MessageProducer producer = session.createProducer(replyTo);
			TextMessage reply = session.createTextMessage(result);
			producer.send(reply);
			//System.out.println("message reply sent, result: " + result);

			producer.close();
			ic.close();

		} catch (Exception e) {
			e.printStackTrace();
			System.out.println("The Message Driven Bean failed!");
		} finally {
			if (conn != null) {
				try {
					conn.close();
				} catch (Exception e) {
					System.out.println("Could not close the connection!" + e);
				}
			}
		}
	}
}
